﻿using System;
using System.Collections.ObjectModel;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO;
using System.Net;
using MsgBox = System.Windows.Forms.MessageBox;

namespace LovciPokladu
{
    class Communicator
    {
        private static string _host;
        private static int _port;
        public static App app;
        public static string[] StatParts;

        public static bool Connect(string host, int port)
        {
            _host = host;
            _port = port;
            app = App.Current as App;

            if (Talk("start"))
            {
                app.Dispatcher.Invoke(() =>
                {
                    app.ServerAddr = _host;
                    app.ServerPort = _port;
                    app.InitialMoves = app.MovesLeft;
                });

                return true;
            }

            return false;
        }

        public static bool Talk(string requestPath)
        {
            System.Diagnostics.Debug.WriteLine(requestPath);

            try
            {
                using (WebClient wc = new WebClient())
                {
                    Uri uri = new Uri("http://" + _host + ":" + _port + "/" + requestPath);
                    return ParseResponse(wc.DownloadString(uri));
                }
            }
            catch (Exception e)
            {
                MsgBox.Show(e.Message, "Komunikace se serverem se nezdařila.");
                return false;
            }
        }

        private static bool ParseResponse(string resp)
        {
            resp = resp.Trim();

            if (resp.StartsWith("Konec hry"))
            {
                string[] scoreParts = resp.Substring(10).Trim().Split(' ');
                string start = String.Empty;

                if (scoreParts.Length == 1)
                {
                    start += "Dosáhli jste " + Int32.Parse(scoreParts[0]) + " bodů.";
                }
                else if (scoreParts.Length == 2)
                {
                    int p1 = Int32.Parse(scoreParts[0]);
                    int p2 = Int32.Parse(scoreParts[1]);

                    if (p1 > p2)
                        start += "Vyhráli jste nad svým protihráčem.";
                    else if (p1 < p2)
                        start += "Protihráč vyhrál :(";
                    else
                        start += "Remíza!";

                    start += "\nNasbírali jste poklad v hodnotě " + p1 + " bodů, váš protihráč v hodnotě " + p2;             
                }

                start += "\n\nSpustit další hru?";

                if (MsgBox.Show(start, "Hra skončila!", System.Windows.Forms.MessageBoxButtons.YesNo, System.Windows.Forms.MessageBoxIcon.Information) == System.Windows.Forms.DialogResult.Yes)
                {
                    App.Current.ShutdownMode = System.Windows.ShutdownMode.OnExplicitShutdown;

                    foreach (var w in Communicator.app.Windows)
                    {
                        if (w is MainWindow)
                        {
                            (w as System.Windows.Window).Close();
                        }
                    }

                    MainWindow mw = new MainWindow();
                    mw.Show();

                    App.Current.ShutdownMode = System.Windows.ShutdownMode.OnMainWindowClose;
                }
                else
                {
                    App.Current.Shutdown();
                }

                return false;

            } else if (resp.StartsWith("Chyba")) {
                MsgBox.Show(resp, "Zpráva od serveru", System.Windows.Forms.MessageBoxButtons.OK, System.Windows.Forms.MessageBoxIcon.Information);
                return false;
            }

            string[] lines = resp.Split('\n');
            string line;
            ObservableCollection<MapField> map = new ObservableCollection<MapField>();
            bool secondPlayerExists = false;

            for (int i = 0; i < lines.Length; i++)
            {
                line = lines[i].Trim();

                if (i == 0)
                {
                    app.Dispatcher.Invoke(() =>
                    {
                        app.GameID = Int32.Parse(line);
                    });
                }
                else if (i == 1)
                {
                    app.Dispatcher.Invoke(() =>
                    {
                        app.MovesLeft = Int32.Parse(line);
                    });
                }
                else if (i >= 2 && i <= 11)
                {
                    for (int j = 0; j + 1 < line.Length; j += 2)
                    {
                        map.Add(new MapField()
                        {
                            Y = i - 2,
                            X = j / 2,
                            Type = line.Substring(j, 2)
                        });
                    }
                }
                else if (i == 12)
                {
                    string[] parts = line.Split(' ');

                    if (parts.Length == 4)
                    {
                        StatParts = parts;

                        int playerX = Int32.Parse(parts[0]) - 1;
                        int playerY = Int32.Parse(parts[1]) - 1;

                        app.Dispatcher.Invoke(() =>
                        {
                            app.PlayerX = playerX;
                            app.PlayerY = playerY;
                            app.FirstHeldTreasure = Int32.Parse(parts[2]);
                            app.FirstScore = Int32.Parse(parts[3]);
                        });

                        map.Add(new MapField()
                        {
                            X = playerX,
                            Y = playerY,
                            Type = "PLAYER"
                        });
                    }
                }
                else if (i == 13)
                {
                    string[] parts = line.Split(' ');

                    if (parts.Length == 4)
                    {
                        StatParts = parts;

                        int playerX = Int32.Parse(parts[0]) - 1;
                        int playerY = Int32.Parse(parts[1]) - 1;
                        secondPlayerExists = true;

                        app.Dispatcher.Invoke(() =>
                        {
                            app.SecondPlayerX = playerX;
                            app.SecondPlayerY = playerY;
                            app.SecondHeldTreasure = Int32.Parse(parts[2]);
                            app.SecondScore = Int32.Parse(parts[3]);
                        });

                        map.Add(new MapField()
                        {
                            X = playerX,
                            Y = playerY,
                            Type = "ENEMY"
                        });
                    }
                }
                else
                {
                    break;
                }
            }

            app.Dispatcher.Invoke(() =>
            {
                app.MapField = map;
                app.SecondPlayerExists = secondPlayerExists;
            });

            return true;                
        }

    }
}
