﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using MsgBox = System.Windows.Forms.MessageBox;

namespace LovciPokladu
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            // už nic :)      
        }

        private void MapGridPresenter_Loaded(object sender, RoutedEventArgs e)
        {
            // po otevreni okna si vyzadej udaje k pripojeni k serveru

            ConnectionDialog cd = new ConnectionDialog();

            if (cd.ShowDialog() == false)
            {
                App.Current.Shutdown();
            }
            else
            {
                MapGridPresenter.DataContext = (Application.Current as App).MapField;

                if ((Application.Current as App).SecondPlayerExists)
                {
                    PlayerTwoStats.Visibility = System.Windows.Visibility.Visible;
                }
                else
                {
                    PlayerTwoStats.Visibility = System.Windows.Visibility.Collapsed;
                }
            }            
        }

        // po kliknuti na tlacitko pohybu
        private void Button_Click(object sender, RoutedEventArgs e)
        {
            if (Communicator.Talk("tah?id=" + (Application.Current as App).GameID + "&pohyb=" + (sender as Button).Tag.ToString()))
            {
                MapGridPresenter.DataContext = (Application.Current as App).MapField;
            }
        }

        private void Window_KeyDown(object sender, KeyEventArgs e)
        {
            // stisknutim sipky na klavesnici simuluj kliknuti na tlacitko

            if (e.Key == Key.Down)
            {
                this.Button_Click(GoDo, new RoutedEventArgs());
            }
            else if (e.Key == Key.Up)
            {
                this.Button_Click(GoUp, new RoutedEventArgs());
            }
            else if (e.Key == Key.Left)
            {
                this.Button_Click(GoLe, new RoutedEventArgs());
            }
            else if (e.Key == Key.Right)
            {
                this.Button_Click(GoRi, new RoutedEventArgs());
            }
        }

        private MapField GetFieldAt(int x, int y)
        {
            foreach (MapField mf in (Application.Current as App).MapField)
            {
                if (mf.X == x && mf.Y == y && !mf.IsPlayer && !mf.IsEnemy)
                {
                    return mf;
                }
            }

            return null;
        }

        enum Move
        {
            Left = 'L',
            Right = 'P',
            Up = 'N',
            Down = 'D'
        }

        private double getDistance(int x1, int x2, int y1, int y2)
        {
            return Math.Pow(x1 - x2, 2) + Math.Pow(y1 - y2, 2);
        }
        
        HashSet<MapField> banlist = new HashSet<MapField>();
        Stack<MapField> history = new Stack<MapField>();

        // hledání cesty přes AI
        private void Button_Click_1(object sender, RoutedEventArgs e)
        {
            string directionChar = "L";

            // nasleduje nemocna implementace A*

            do {
                MapGridPresenter.DataContext = (Application.Current as App).MapField;
                Dictionary<Move, Tuple<int, int>> moveScore = new Dictionary<Move, Tuple<int, int>>();
                Dictionary<MapField, Tuple<double, int>> treasures = new Dictionary<MapField, Tuple<double, int>>();
            
                MapField shortestPathTo = null; // i.e. where we're going (shortest distance)

                int px = (Application.Current as App).PlayerX;
                int py = (Application.Current as App).PlayerY;
                MapField currentField = GetFieldAt(px, py);

                if (currentField.IsFriendlyBase)
                {
                    history.Clear();
                    banlist.Clear(); // jsme doma - zacni od zacatku
                }

                history.Push(currentField);

                MapField home = null;

                foreach (MapField mf in (Application.Current as App).MapField)
                {
                    if (mf.IsTreasure) {
                        treasures.Add(mf, new Tuple<double, int>(getDistance(mf.X, px, mf.Y, py), mf.TreasureValue));

                        if (shortestPathTo == null || treasures[shortestPathTo].Item1 > treasures[mf].Item1)
                            shortestPathTo = mf;
                    }
                    else if (mf.IsFriendlyBase)
                    {
                        home = mf;
                    }
                }

                if ((Application.Current as App).FirstHeldTreasure > 0)
                {
                    shortestPathTo = home; // pokud mame v ruce poklad, jdeme domu, ne k dalsimu pokladu
                }

                Move direction = Move.Left;

                if (shortestPathTo == null)
                {
                    MsgBox.Show("Nevím kam jít :(");
                    continue;
                }
                else
                {
                    MapField f = null;
                    double dist = Double.MaxValue;
                    double loopDist;
                    MapField next = null;

                    for (int xd = -1; xd <= 1; xd += 2) // zjisteni moznosti pohybu na ose X
                    {
                        if (px + xd < 0 || px + xd >= 10)
                            continue;

                        f = GetFieldAt(px + xd, py);

                        if (!f.IsWall && !f.IsEnemyBase && !banlist.Contains(f) && !history.Contains(f))
                        { // if (can move to this field)
                            loopDist = getDistance(shortestPathTo.X, f.X, shortestPathTo.Y, f.Y);

                            if (loopDist < dist)
                            {
                                dist = loopDist;
                                next = f;
                            }
                        }
                    }

                    for (int yd = -1; yd <= 1; yd += 2) // zjisteni moznosti pohybu na ose Y
                    {
                        if (py + yd < 0 || py + yd >= 10)
                            continue;

                        f = GetFieldAt(px, py + yd);

                        if (!f.IsWall && !f.IsEnemyBase && !banlist.Contains(f) && !history.Contains(f))
                        { // if (can move to this field)
                            loopDist = getDistance(shortestPathTo.X, f.X, shortestPathTo.Y, f.Y);

                            if (loopDist < dist)
                            {
                                dist = loopDist;
                                next = f;
                            }
                        }
                    }

                    if (next == null) // zadna mozna cesta neni, vrat se o krok zpet
                    {
                        MsgBox.Show("Žádná cesta, jdu zpět.");
                        banlist.Add(currentField);
                        history.Pop();
                        next = history.Pop();
                    }
                    else
                    {
                        if (next.X < currentField.X)
                        {
                            direction = Move.Left;
                        }
                        else if (next.X > currentField.X)
                        {
                            direction = Move.Right;
                        }
                        else if (next.Y < currentField.Y)
                        {
                            direction = Move.Up;
                        }
                        else if (next.Y > currentField.Y)
                        {
                            direction = Move.Down;
                        }
                    }
                }

                // nastav smer pohybu pro komunikator
                directionChar = "L";
                if (direction == Move.Right)
                    directionChar = "P";
                else if (direction == Move.Up)
                    directionChar = "N";
                else if (direction == Move.Down)
                    directionChar = "D";

            }
            while (Communicator.Talk("tah?id=" + (Application.Current as App).GameID + "&pohyb=" + directionChar));
        }
    }
}
